/******************************************
  PURPOSE:  Create a remote controlled stepper motor
  Created by      Clement Butler
  DATE:   4/12/2021
*******************************************/

#include <Stepper.h> //header file to incorporate the stepper motor into the arduino sketch
#include <IRremote.h> //header file to incorporate the IR sensor into the arduino sketch
//Version 3.1.0 of the IRremote library will be needed for the code to work

#define DECODE_NEC //defining the necessary code needed to decode the incoming signal from the remote
#define IR_RECEIVE_PIN 2 //pin number on the arduino for the input signla from the IR sensor
#define delayTime 8 //delay time in milliseconds between each tooth motion of the stepper motor - 8 ms being the shortest time allowable 

const int defaultSteps = 16;
int stepNumber = defaultSteps;  //number of steps the motor will take with each activation
int stepsPerRevolution = 100;  //number of steps per revolution for the stepper motor

int motorPin1 = 8; //defining a pin number for the stepper motor
int motorPin2 = 9; //defining a pin number for the stepper motor
int motorPin3 = 10; //defining a pin number for the stepper motor
int motorPin4 = 11; //defining a pin number for the stepper motor

#define buttonRight 3208707840 //signal for the "NEXT" button on the remote
#define buttonLeft 3141861120 //signal for the "PREV" arrow button on the remote
#define buttonMinusSpeed 3125149440 //signal for the "CH-" button on the remote
#define buttonBaseSpeed 3108437760 //signal for the "CH" button on the remote
#define buttonPlusSpeed 3091726080 //signal for the "CH+" button on the remote
#define buttonPlus 3927310080 //signal for the "VOL+" button on the remote
#define buttonMinus 4161273600 //signal for the "VOL-" button on the remote
#define buttonPause 3158572800 //signal for the "PLAY/PAUSE" button on the remote
#define buttonEqualize 4127850240 //signal for the "EQ" button on the remote
#define buttonShortAngle 3860463360 //signal for the "100+" button on the remote
#define buttonWideAngle 4061003520 //signal for the "200+" button on the remote
#define buttonRepeat 0 //signal for if a button is being held down

#define buttonOne 4077715200 //signal for the "1" button on the remote
#define buttonTwo 3877175040 //signal for the "2" button on the remote
#define buttonThree 2707357440 //signal for the "3" button on the remote
#define buttonFour 4144561920 //signal for the "4" button on the remote
#define buttonFive 3810328320 //signal for the "5" button on the remote
#define buttonSix 2774204160 //signal for the "6" button on the remote
#define buttonSeven 3175284480 //signal for the "7" button on the remote
#define buttonEight 2907897600 //signal for the "8" button on the remote
#define buttonNine 3041591040 //signal for the "9" button on the remote

int directionVariable = 0; //variable to provide a safety measure for the loop below
int limit = 1; //variable to assist with the iterations function in the loop
int span = 1; //variable to assist with controlling the angle on the equal proportion spin

void setup() {
  Serial.begin(115200);
  IrReceiver.begin(IR_RECEIVE_PIN, ENABLE_LED_FEEDBACK, USE_DEFAULT_FEEDBACK_LED_PIN); // Start the receiver
  pinMode(motorPin1,OUTPUT); //setting the pin for the stepper motor as an output
  pinMode(motorPin2,OUTPUT); //setting the pin for the stepper motor as an output
  pinMode(motorPin3,OUTPUT); //setting the pin for the stepper motor as an output
  pinMode(motorPin4,OUTPUT); //setting the pin for the stepper motor as an output
}

void loop() {
  if (IrReceiver.decode()){//has a transmission been received?
    Serial.println(IrReceiver.decodedIRData.decodedRawData);//prints out the incoming signal in the serial monitor
    Serial.println(span);
    //Functions for a clockwise turn---------------------------------------------------------------------------------------------
    if (IrReceiver.decodedIRData.decodedRawData == buttonLeft){//did the sensor pick up the signal for the left button on the remote being pressed?
      directionVariable = 1;//to give a unique value to the sensor to ensure a clockwise turn
    }
    if (IrReceiver.decodedIRData.decodedRawData == buttonRepeat && directionVariable == 1){//is the left button/"PREV" button being held down?
      forward();//rotate the stepper motor forward or in clockwise direction
    }
    
    //Functions for a counterclockwise turn-------------------------------------------------------------------------------------- 
    if(IrReceiver.decodedIRData.decodedRawData == buttonRight){//did the sensor pick up the signal for the right button on the remote being pressed?
      directionVariable = 2;//to give a unique value to the sensor to ensure a counterclockwise turn
    }
    if (IrReceiver.decodedIRData.decodedRawData == buttonRepeat && directionVariable == 2){//is the right button/"NEXT" button being held down?
      backward();//rotate the stepper motor backward or in counterclockwise direction
    }

    //Functions to return to base speed------------------------------------------------------------------------------------------
    if (IrReceiver.decodedIRData.decodedRawData == buttonBaseSpeed){//did the "CH" button/base speed button get pressed?
      directionVariable = 3;//to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
      stepNumber = defaultSteps;//returns the step number back to initial value which will return the speed to normal
      //defauleSteps set to 30
    }

    //Functions to spin counterclockwise continuously----------------------------------------------------------------------------
    if (IrReceiver.decodedIRData.decodedRawData == buttonPlus){//did the "VOL+" button/counterclockwise spin button get pressed?
      directionVariable = 3;//to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
      stepNumber = 515; //increases the number of steps necessary to create the full rotation
      for (int sweep=0;sweep<limit;sweep++){
        backward(); //rotate the stepper motor backward or in counterclockwise direction
      }
      stepNumber = defaultSteps; //returns the number of steps back to its default value
    }

    //Functions to spin clockwise continuously-----------------------------------------------------------------------------------
    if (IrReceiver.decodedIRData.decodedRawData == buttonMinus){//did the "VOL-" button/clockwise spin button get pressed?
      directionVariable = 3;//to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
      stepNumber = 515; //increases the number of steps necessary to create the full rotation
      for (int sweep=0;sweep<limit;sweep++){
        forward(); //rotate the stepper motor forward or in clockwise direction
      }
      stepNumber = defaultSteps; //returns the number of steps back to its default value
    }

    //Functions to spin both directions in equal proportions---------------------------------------------------------------------
    if (IrReceiver.decodedIRData.decodedRawData == buttonEqualize){//did the "EQ" button/equal spin button get pressed?
      directionVariable = 3;//to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
      if (span == 1){
        stepNumber = 87; //increases the number of steps necessary for the first stage of the short angle surveilance
        backward(); //rotate the stepper motor backward or in counterclockwise direction
        stepNumber = 2*stepNumber; //doubles steps necessary for the rest of the short angle surveilance
        for (int sweep=0;sweep<limit;sweep++){
          forward(); //rotate the stepper motor forward or in clockwise direction
          backward(); //rotate the stepper motor backward or in counterclockwise direction
        }
        stepNumber = 0.5*stepNumber; //halves the number of steps so that the sequence may come to an end at the same position it started at
        forward(); //rotate the stepper motor forward or in clockwise direction
        stepNumber = defaultSteps; //rotate the stepper motor forward or in clockwise direction
      }

      if (span == 2){
        stepNumber = 175; //increases the number of steps necessary for the first stage of the wide angle surveilance
        backward(); //rotate the stepper motor backward or in counterclockwise direction
        stepNumber = 2*stepNumber; //doubles steps necessary for the rest of the short angle surveilance
        for (int sweep=0;sweep<limit;sweep++){
          forward(); //rotate the stepper motor forward or in clockwise direction
          backward(); //rotate the stepper motor backward or in counterclockwise direction
        }
        stepNumber = 0.5*stepNumber; //halves the number of steps so that the sequence may come to an end at the same position it started at
        forward(); //rotate the stepper motor forward or in clockwise direction
        stepNumber = defaultSteps; //rotate the stepper motor forward or in clockwise direction
      }
    }
    
    //Functions to control the number of iterations--------------------------------------------------------------------------
    if (IrReceiver.decodedIRData.decodedRawData == buttonOne){
      limit = 1; //sets the number of iterations allowable to ONE
      directionVariable = 3; //to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
    }
    if (IrReceiver.decodedIRData.decodedRawData == buttonTwo){
      limit = 2; //sets the number of iterations allowable to TWO
      directionVariable = 3; //to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
    }
    if (IrReceiver.decodedIRData.decodedRawData == buttonThree){
      limit = 3; //sets the number of iterations allowable to THREE
      directionVariable = 3; //to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
    }
    if (IrReceiver.decodedIRData.decodedRawData == buttonFour){
      limit = 4; //sets the number of iterations allowable to FOUR
      directionVariable = 3; //to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
    }
    if (IrReceiver.decodedIRData.decodedRawData == buttonFive){
      limit = 5; //sets the number of iterations allowable to FIVE
      directionVariable = 3; //to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
    }
    if (IrReceiver.decodedIRData.decodedRawData == buttonSix){
      limit = 6; //sets the number of iterations allowable to SIX
      directionVariable = 3; //to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
    }
    if (IrReceiver.decodedIRData.decodedRawData == buttonSeven){
      limit = 7; //sets the number of iterations allowable to SEVEN
      directionVariable = 3; //to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
    }
    if (IrReceiver.decodedIRData.decodedRawData == buttonEight){
      limit = 8; //sets the number of iterations allowable to EIGHT
      directionVariable = 3; //to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
    }
    if (IrReceiver.decodedIRData.decodedRawData == buttonNine){
      limit = 9; //sets the number of iterations allowable to NINE
      directionVariable = 3; //to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
    }

    //Functions to control the field of view angle for the surveilance------------------------------------------------------
    if (IrReceiver.decodedIRData.decodedRawData == buttonShortAngle){
      span = 1; //sets the angle to a short angle for the field of view
      directionVariable = 3; //to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
    }
    if (IrReceiver.decodedIRData.decodedRawData == buttonWideAngle){
      span = 2; //sets the angle to a short angle for the field of view
      directionVariable = 3; //to give a unique value to the sensor so that the mototr doesn't spin if this button is held down
    }
  } 
  IrReceiver.resume(); //sensor ready to receive the next value     
}
  
